(function(module) {
	'use strict'

	const async = module.parent.require('async')
	const nconf = module.parent.require('nconf')
	const validator = module.parent.require('validator')
	const fs = require('fs')
	const path = require('path')
	const user = module.parent.require('./user')
	const topics = module.parent.require('./topics')
	const translator = module.parent.require('../public/src/modules/translator')
	const templates = module.parent.require('templates.js')
	let app

	const Widget = {
		templates: {}
	}

  /**
   * Init loads templates that we need and put them in a reusable list
   * @param  {Object}   params   App params
   * @param  {Function} callback
   */
	Widget.init = (params, callback) => {
		app = params.app

		var templatesToLoad = [
      'widgets/kudos.tpl',
      'widgets/senaste.tpl',
      'widgets/tags.tpl',

      'admin/kudos.tpl',
			'admin/senaste.tpl',
      'admin/tags.tpl'
		]

		function loadTemplate (template, next) {
      var templatePath = path.resolve(__dirname, './public/templates/' + template)

			fs.readFile(templatePath, (err, data) => {
				if (err) {
					return next(err)
				}

				Widget.templates[template] = data.toString()
				next(null)
			})
		}

		async.each(templatesToLoad, loadTemplate)

		callback()
	}

  /**
   * Add general properties to all widgets
   * @param  {Object} widget - Widget to extend
   * @return {Object} Extended widget
   */
  const generalProperties = (widget) => {
    return {
      title: widget.data.title,
      relative_path: nconf.get('relative_path')
    }
  }

  /**
   * Helper function for rendering a template
   * @param  {String}   template   Template path
   * @param  {Object}   properties Properties to include in template
   * @param  {Function} cb         callback
   */
  const render = (template, properties, cb) => {
    app.render(template, properties, (err, html) => {
      translator.translate(html, translatedHTML => {
        cb(err, translatedHTML)
      })
    })
  }

  /**
   * Vimla widget - Render a list sorted DESC of users kudos/reputation
   * @param  {Object}   widget   Widget settings
   * @param  {Function} callback
   */
  Widget.renderMostKudos = (widget, callback) => {
    var numUsers = widget.data.numUsers || 5

    user.getUsersFromSet('users:reputation', widget.uid, 0, numUsers, (err, data) => {
      if (err) {
        return callback(err)
      }

      // Remove users without reputation
      data = data.filter(user => user.reputation > 0)

      const properties = Object.assign({}, {
        users: data,
        numUsers: numUsers
      }, generalProperties(widget))

      render('widgets/kudos', properties, callback)
    })
  }

  /**
   * Vimla widget - Render a list of the most used tags
   * @param  {Object}   widget   Widget settings
   * @param  {Function} callback
   */
  Widget.renderTags = (widget, callback) => {
    const numTags = widget.data.numTags || 8

    topics.getTags(0, numTags - 1, (err, tags) => {
      if (err) {
        return callback(err)
      }

      const properties = Object.assign({}, {
        tags: tags
      }, generalProperties(widget))

      render('widgets/tags', properties, callback)
    })
  }

  /**
   * Vimla widget - Render a list of the latest posts
   * @param  {Object}   widget   Widget settings
   * @param  {Function} callback
   */
	Widget.renderVimlaSenaste = (widget, callback) => {
		var numTopics = (widget.data.numTopics || 8) - 1
    var max = Math.max(0, numTopics)
    var uid = widget.uid

		topics.getTopicsFromSet('topics:recent', uid, 0, max, (err, data) => {
			if (err) {
				return callback(err)
			}

      const properties = Object.assign({}, {
        topics: data.topics,
        numTopics: numTopics,
      }, generalProperties(widget))

      render('widgets/senaste', properties, callback)
		})
	}

  /**
   * Define the widgets to display them in the ACP
   * @param  {Object}   widget   Widget settings
   * @param  {Function} callback
   */
	Widget.defineWidgets = (widgets, callback) => {
		widgets = widgets.concat([
      {
  			widget: 'vimlaSenaste',
  			name: 'Vimla - Senaste inlägg',
  			description: 'Visa de senaste inläggen',
  			content: Widget.templates['admin/senaste.tpl']
  		},
      {
        widget: 'vimlaKudos',
        name: 'Vimla - Kudostopplista',
        description: 'Topplista',
        content: Widget.templates['admin/kudos.tpl']
      },
      {
        widget: 'vimlaTags',
        name: 'Vimla - Taggar',
        description: 'Bästa taggarna',
        content: Widget.templates['admin/tags.tpl']
      }
    ])

    callback(null, widgets)
	}

	module.exports = Widget
}(module))
